@@ -12,7 +12,7 @@
 #include "mpg123lib_intern.h" /* includes optimize.h */
 #include "debug.h"
 
-#if ((defined OPT_X86) || (defined OPT_X86_64)) && (defined OPT_MULTI)
+#if ((defined OPT_X86) || (defined OPT_X86_64) || (defined OPT_NEON)) && (defined OPT_MULTI)
 #include "getcpuflags.h"
 static struct cpuflags cpu_flags;
 #else
@@ -28,6 +28,7 @@ static struct cpuflags cpu_flags;
 #define cpu_sse2(s)     1
 #define cpu_sse3(s)     1
 #define cpu_avx(s)      1
+#define cpu_neon(s)     1
 #endif
 
 /* Ugly macros to build conditional synth function array values. */
@@ -38,6 +39,8 @@ static struct cpuflags cpu_flags;
 #define IF8(synth)
 #endif
 
+#ifndef NO_SYNTH32
+
 #ifndef NO_REAL
 #define IFREAL(synth) synth,
 #else
@@ -50,6 +53,13 @@ static struct cpuflags cpu_flags;
 #define IF32(synth)
 #endif
 
+#else
+
+#define IFREAL(synth)
+#define IF32(synth)
+
+#endif
+
 #ifndef NO_16BIT
 #	define OUT_SYNTHS(synth_16, synth_8, synth_real, synth_32) { synth_16, IF8(synth_8) IFREAL(synth_real) IF32(synth_32) }
 #else
@@ -156,6 +166,7 @@ static int find_synth(func_synth synth,  const func_synth synths[r_limit][f_limi
 }
 
 
+#if defined(OPT_SSE) || defined(OPT_SSE_VINTAGE)
 /* After knowing that it is either vintage or current SSE,
    this separates the two. In case of non-OPT_MULTI, only one
    of OPT_SSE and OPT_SSE_VINTAGE is active. */
@@ -171,6 +182,7 @@ static enum optdec sse_or_vintage(mpg123_handle *fr)
 #	endif
 	return type;
 }
+#endif
 
 /* Determine what kind of decoder is actually active
    This depends on runtime choices which may cause fallback to i386 or generic code. */
@@ -257,6 +269,8 @@ static int find_dectype(mpg123_handle *fr)
 #endif
 #endif /* 16bit */
 
+#ifndef NO_SYNTH32
+
 #ifndef NO_REAL
 #if defined(OPT_SSE) || defined(OPT_SSE_VINTAGE)
 	else if(basic_synth == synth_1to1_real_sse)
@@ -300,6 +314,8 @@ static int find_dectype(mpg123_handle *fr)
 #endif
 #endif /* 32bit */
 
+#endif /* any 32 bit synth */
+
 #ifdef OPT_X86
 	else if(find_synth(basic_synth, plain_i386))
 	type = idrei;
@@ -342,20 +358,20 @@ int set_synth_functions(mpg123_handle *fr)
 	/* Select the basic output format, different from 16bit: 8bit, real. */
 	if(FALSE){}
 #ifndef NO_16BIT
-	else if(fr->af.encoding & MPG123_ENC_16)
+	else if(fr->af.dec_enc & MPG123_ENC_16)
 	basic_format = f_16;
 #endif
 #ifndef NO_8BIT
-	else if(fr->af.encoding & MPG123_ENC_8)
+	else if(fr->af.dec_enc & MPG123_ENC_8)
 	basic_format = f_8;
 #endif
 #ifndef NO_REAL
-	else if(fr->af.encoding & MPG123_ENC_FLOAT)
+	else if(fr->af.dec_enc & MPG123_ENC_FLOAT)
 	basic_format = f_real;
 #endif
 #ifndef NO_32BIT
 	/* 24 bit integer means decoding to 32 bit first. */
-	else if(fr->af.encoding & MPG123_ENC_32 || fr->af.encoding & MPG123_ENC_24)
+	else if(fr->af.dec_enc & MPG123_ENC_32 || fr->af.dec_enc & MPG123_ENC_24)
 	basic_format = f_32;
 #endif
 
@@ -759,23 +775,6 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 	}
 #endif
 
-#ifdef OPT_GENERIC_DITHER
-	if(!done && (auto_choose || want_dec == generic_dither))
-	{
-		chosen = "dithered generic";
-		fr->cpu_opts.type = generic_dither;
-		dithered = TRUE;
-#		ifndef NO_16BIT
-		fr->synths.plain[r_1to1][f_16] = synth_1to1_dither;
-#		ifndef NO_DOWNSAMPLE
-		fr->synths.plain[r_2to1][f_16] = synth_2to1_dither;
-		fr->synths.plain[r_4to1][f_16] = synth_4to1_dither;
-#		endif
-#		endif
-		done = 1;
-	}
-#endif
-
 #	ifdef OPT_ALTIVEC
 	if(!done && (auto_choose || want_dec == altivec))
 	{
@@ -798,7 +797,7 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 #	endif
 
 #	ifdef OPT_NEON
-	if(!done && (auto_choose || want_dec == neon))
+	if(!done && (auto_choose || want_dec == neon) && cpu_neon(cpu_flags))
 	{
 		chosen = dn_neon;
 		fr->cpu_opts.type = neon;
@@ -839,6 +838,23 @@ int frame_cpu_opt(mpg123_handle *fr, const char* cpu)
 	}
 #	endif
 
+#ifdef OPT_GENERIC_DITHER
+	if(!done && (auto_choose || want_dec == generic_dither))
+	{
+		chosen = "dithered generic";
+		fr->cpu_opts.type = generic_dither;
+		dithered = TRUE;
+#		ifndef NO_16BIT
+		fr->synths.plain[r_1to1][f_16] = synth_1to1_dither;
+#		ifndef NO_DOWNSAMPLE
+		fr->synths.plain[r_2to1][f_16] = synth_2to1_dither;
+		fr->synths.plain[r_4to1][f_16] = synth_4to1_dither;
+#		endif
+#		endif
+		done = 1;
+	}
+#endif
+
 	fr->cpu_opts.class = decclass(fr->cpu_opts.type);
 
 #	ifndef NO_8BIT
@@ -1024,7 +1040,7 @@ void check_decoders(void )
 	return;
 #else
 	const char **d = mpg123_supported_decoder_list;
-#if (defined OPT_X86) || (defined OPT_X86_64)
+#if (defined OPT_X86) || (defined OPT_X86_64) || (defined OPT_NEON)
 	getcpuflags(&cpu_flags);
 #endif
 #ifdef OPT_X86
@@ -1082,7 +1098,7 @@ void check_decoders(void )
 	*(d++) = dn_arm;
 #endif
 #ifdef OPT_NEON
-	*(d++) = dn_neon;
+	if(cpu_neon(cpu_flags)) *(d++) = dn_neon;
 #endif
 #ifdef OPT_GENERIC
 	*(d++) = dn_generic;
