@@ -141,6 +141,7 @@ static int filept = -1;
 
 static int network_sockets_used = 0; /* Win32 socket open/close Support */
 
+char *fullprogname = NULL; /* Copy of argv[0]. */
 char *binpath; /* Path to myself. */
 
 /* File-global storage of command line arguments.
@@ -175,7 +176,7 @@ void next_track(void)
 void prev_track(void)
 {
 	if(pl.pos > 2) pl.pos -= 2;
-	else pl.pos = 0;
+	else pl.pos = skip_tracks = 0;
 
 	next_track();
 }
@@ -205,6 +206,7 @@ void safe_exit(int code)
 #endif
 	/* It's ugly... but let's just fix this still-reachable memory chunk of static char*. */
 	split_dir_file("", &dummy, &dammy);
+	if(fullprogname) free(fullprogname);
 	exit(code);
 }
 
@@ -658,10 +660,13 @@ int play_frame(void)
 {
 	unsigned char *audio;
 	int mc;
+	long new_header = 0;
 	size_t bytes;
 	debug("play_frame");
 	/* The first call will not decode anything but return MPG123_NEW_FORMAT! */
 	mc = mpg123_decode_frame(mh, &framenum, &audio, &bytes);
+	mpg123_getstate(mh, MPG123_FRESH_DECODER, &new_header, NULL);
+
 	/* Play what is there to play (starting with second decode_frame call!) */
 	if(bytes)
 	{
@@ -702,15 +707,16 @@ int play_frame(void)
 			mpg123_getformat(mh, &rate, &channels, &format);
 			if(param.verbose > 2) fprintf(stderr, "\nNote: New output format %liHz %ich, format %i\n", rate, channels, format);
 
-			if(!param.quiet)
-			{
-				fprintf(stderr, "\n");
-				if(param.verbose) print_header(mh);
-				else print_header_compact(mh);
-			}
+			new_header = 1;
 			reset_audio(rate, channels, format);
 		}
 	}
+	if(new_header && !param.quiet)
+	{
+		fprintf(stderr, "\n");
+		if(param.verbose) print_header(mh);
+		else print_header_compact(mh);
+	}
 	return 1;
 }
 
@@ -806,17 +812,23 @@ int main(int sys_argc, char ** sys_argv)
 	win32_net_init();
 #endif
 
+	if(!(fullprogname = strdup(argv[0])))
+	{
+		error("OOM"); /* Out Of Memory. Don't waste bytes on that error. */
+		safe_exit(1);
+	}
 	/* Extract binary and path, take stuff before/after last / or \ . */
-	if((prgName = strrchr(argv[0], '/')) || (prgName = strrchr(argv[0], '\\')))
+	if(  (prgName = strrchr(fullprogname, '/')) 
+	  || (prgName = strrchr(fullprogname, '\\')))
 	{
 		/* There is some explicit path. */
 		prgName[0] = 0; /* End byte for path. */
 		prgName++;
-		binpath = argv[0];
+		binpath = fullprogname;
 	}
 	else
 	{
-		prgName = argv[0]; /* No path separators there. */
+		prgName = fullprogname; /* No path separators there. */
 		binpath = NULL; /* No path at all. */
 	}
 
@@ -1066,10 +1078,13 @@ int main(int sys_argc, char ** sys_argv)
 #ifdef HAVE_TERMIOS
 			/* We need the opportunity to cancel in case of --loop -1 . */
 			if(param.term_ctrl) term_control(mh, ao);
+			else
 #endif
 			/* No wait for a second interrupt before we started playing. */
 			if(intflag) break;
 
+			/* We already interrupted this cycle, start fresh with the next one. */
+			intflag = FALSE;
 			continue;
 		}
 
@@ -1202,7 +1217,7 @@ int main(int sys_argc, char ** sys_argv)
 	}
 	else if(param.verbose) fprintf(stderr, "\n");
 
-	mpg123_close(mh);
+	close_track();
 
 	if (intflag)
 	{
